@@ -21,9 +21,6 @@ class MX_GENGLSL_API LightNodeGlsl : public GlslImplementation
     void createVariables(const ShaderNode& node, GenContext& context, Shader& shader) const override;
 
     void emitFunctionCall(const ShaderNode& node, GenContext& context, ShaderStage& stage) const override;
-
-  private:
-    mutable ClosureContext _callEmission;
 };
 
 MATERIALX_NAMESPACE_END
