@@ -24,7 +24,7 @@ class MX_GENMDL_API BlurNodeMdl : public BlurNode
     void emitSamplingFunctionDefinition(const ShaderNode& node, GenContext& context, ShaderStage& stage) const override;
 
     /// Output sample array
-    void outputSampleArray(const ShaderGenerator& shadergen, ShaderStage& stage, const TypeDesc* inputType,
+    void outputSampleArray(const ShaderGenerator& shadergen, ShaderStage& stage, TypeDesc inputType,
                            const string& sampleName, const StringVec& sampleStrings) const override;
 };
 
