@@ -23,8 +23,8 @@ class MX_GENMDL_API HeightToNormalNodeMdl : public ConvolutionNode
     const string& getTarget() const override;
 
   protected:
-    /// Return if given type is an acceptible input
-    bool acceptsInputType(const TypeDesc* type) const override;
+    /// Return if given type is an acceptable input
+    bool acceptsInputType(TypeDesc type) const override;
 
     /// Compute offset strings for sampling
     void computeSampleOffsetStrings(const string& sampleSizeName, const string& offsetTypeString,
